import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.nio.file.*;

public class MinecraftFontGenerator extends JFrame {
    private JTextField filePathField;
    private JTextField shift1Field;
    private JTextField shift2Field;
    private JTextField sizeField;
    private JTextField oversampleField;
    private File selectedFile;

    private static final String PACK_DICT_TEMPLATE = 
        "{\n" +
        "    \"providers\": [\n" +
        "        {\n" +
        "            \"type\": \"ttf\",\n" +
        "            \"file\": \"minecraft:font.ttf\",\n" +  // 固定文件名
        "            \"shift\": [%s, %s],\n" +
        "            \"size\": %s,\n" +
        "            \"oversample\": %s\n" +
        "        }\n" +
        "    ]\n" +
        "}";

    private static final String PACK_META = 
        "{\n" +
        "  \"pack\": {\n" +
        "    \"pack_format\": 4,\n" +
        "    \"description\": \"generated by minecraft font resourcepack generator\"\n" +
        "  }\n" +
        "}";

    public MinecraftFontGenerator() {
        initUI();
    }

    private void initUI() {
        setTitle("Minecraft 字体资源包生成器");
        setSize(500, 300);
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setLocationRelativeTo(null);

        // 主面板布局
        JPanel mainPanel = new JPanel(new BorderLayout(10, 10));
        mainPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        // 文件选择面板
        JPanel filePanel = createFilePanel();
        mainPanel.add(filePanel, BorderLayout.NORTH);

        // 参数输入面板
        JPanel inputPanel = createInputPanel();
        mainPanel.add(inputPanel, BorderLayout.CENTER);

        // 生成按钮
        JButton generateButton = new JButton("生成资源包");
        generateButton.addActionListener(e -> generatePack());
        mainPanel.add(generateButton, BorderLayout.SOUTH);

        add(mainPanel);
    }

    private JPanel createFilePanel() {
        JPanel panel = new JPanel(new BorderLayout(5, 5));
        JButton browseButton = new JButton("选择字体文件");
        filePathField = new JTextField();
        filePathField.setEditable(false);

        browseButton.addActionListener(e -> browseFile());
        
        panel.add(browseButton, BorderLayout.WEST);
        panel.add(filePathField, BorderLayout.CENTER);
        return panel;
    }

    private JPanel createInputPanel() {
        JPanel panel = new JPanel(new GridLayout(4, 2, 5, 5));
        
        shift1Field = new JTextField("0");
        shift2Field = new JTextField("0");
        sizeField = new JTextField("11");
        oversampleField = new JTextField("2");

        panel.add(new JLabel("横向偏移:"));
        panel.add(shift1Field);
        panel.add(new JLabel("竖向偏移:"));
        panel.add(shift2Field);
        panel.add(new JLabel("字体大小:"));
        panel.add(sizeField);
        panel.add(new JLabel("解析度:"));
        panel.add(oversampleField);

        return panel;
    }

    private void browseFile() {
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileFilter(new javax.swing.filechooser.FileNameExtensionFilter("TTF 字体文件", "ttf"));
        
        int result = fileChooser.showOpenDialog(this);
        if (result == JFileChooser.APPROVE_OPTION) {
            selectedFile = fileChooser.getSelectedFile();
            filePathField.setText(selectedFile.getAbsolutePath());
            copyToCache();
        }
    }

    private void copyToCache() {
        try {
            Path cacheDir = Paths.get("cache");
            if (!Files.exists(cacheDir)) {
                Files.createDirectories(cacheDir);
            }
            Files.copy(selectedFile.toPath(),
                      cacheDir.resolve("font.ttf"),
                      StandardCopyOption.REPLACE_EXISTING);
        } catch (IOException ex) {
            showError("文件复制失败: " + ex.getMessage());
        }
    }

    private void generatePack() {
        try {
            if (!validateInputs()) return;

            String packName = showInputDialog("请输入资源包名称（默认test）:", "test");
            if (packName == null) return;

            createResourcePack(packName);
            showSuccess("资源包已生成在: generated/" + packName);
            openOutputDirectory(packName);

        } catch (Exception ex) {
            showError("生成失败: " + ex.getMessage());
        }
    }

    private boolean validateInputs() {
        if (selectedFile == null) {
            showWarning("请先选择字体文件！");
            return false;
        }
        
        if (!isValidNumber(shift1Field.getText()) ||
            !isValidNumber(shift2Field.getText()) ||
            !isValidNumber(sizeField.getText()) ||
            !isValidNumber(oversampleField.getText())) {
            showWarning("所有输入值必须是数字！");
            return false;
        }
        
        return true;
    }

    private void createResourcePack(String packName) throws IOException {
        // 构建完整路径
        Path packRoot = Paths.get("generated", packName);
        Path assetsDir = packRoot.resolve("assets/minecraft/font");

        // 创建目录结构
        Files.createDirectories(assetsDir);

        // 生成配置文件内容
        String packDict = String.format(PACK_DICT_TEMPLATE,
                shift1Field.getText(),
                shift2Field.getText(),
                sizeField.getText(),
                oversampleField.getText());

        // 写入文件
        Files.writeString(packRoot.resolve("pack.mcmeta"), PACK_META);
        Files.writeString(assetsDir.resolve("default.json"), packDict);
        Files.copy(Paths.get("cache/font.ttf"), 
                 assetsDir.resolve("font.ttf"),
                 StandardCopyOption.REPLACE_EXISTING);
    }

    // 以下是工具方法
    private boolean isValidNumber(String input) {
        try {
            Double.parseDouble(input);
            return true;
        } catch (NumberFormatException e) {
            return false;
        }
    }

    private String showInputDialog(String message, String defaultValue) {
        String input = JOptionPane.showInputDialog(this, message, defaultValue);
        return (input == null || input.trim().isEmpty()) ? defaultValue : input.trim();
    }

    private void showError(String message) {
        JOptionPane.showMessageDialog(this, message, "错误", JOptionPane.ERROR_MESSAGE);
    }

    private void showWarning(String message) {
        JOptionPane.showMessageDialog(this, message, "警告", JOptionPane.WARNING_MESSAGE);
    }

    private void showSuccess(String message) {
        JOptionPane.showMessageDialog(this, message, "完成", JOptionPane.INFORMATION_MESSAGE);
    }

    private void openOutputDirectory(String packName) {
        try {
            Path path = Paths.get("generated", packName);
            if (Desktop.isDesktopSupported()) {
                Desktop.getDesktop().open(path.toFile());
            }
        } catch (IOException ex) {
            showError("无法打开输出目录: " + ex.getMessage());
        }
    }

    public static void main(String[] args) {
        EventQueue.invokeLater(() -> {
            try {
                UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            } catch (Exception ignored) {}
            
            MinecraftFontGenerator app = new MinecraftFontGenerator();
            app.setVisible(true);
        });
    }
}